--- channels/chan_sip.c	2013-01-09 20:30:25.000000000 +0000
+++ channels/chan_sip.c	2013-02-20 12:35:40.865451592 +0000
@@ -4595,14 +4595,14 @@
 	return res;
 }
 
+/*! \brief Completely (re)init DSP
+ *
+ *  \note Can (re)enable temporarily disabled with update_dsp_detect()
+ *  function features, e.g. for sndfax_exec() and rcvfax_exec() */
 static void enable_dsp_detect(struct sip_pvt *p)
 {
 	int features = 0;
 
-	if (p->dsp) {
-		return;
-	}
-
 	if ((ast_test_flag(&p->flags[0], SIP_DTMF) == SIP_DTMF_INBAND) ||
 	    (ast_test_flag(&p->flags[0], SIP_DTMF) == SIP_DTMF_AUTO)) {
 		if (p->rtp) {
@@ -4619,7 +4619,7 @@
 		return;
 	}
 
-	if (!(p->dsp = ast_dsp_new())) {
+	if (!p->dsp && !(p->dsp = ast_dsp_new())) {
 		return;
 	}
 
@@ -4629,6 +4629,36 @@
 	}
 }
 
+/*! \brief Update detect features for already enabled DSP */
+static void update_dsp_detect(struct sip_pvt *p, int onoff, int digit, int fax)
+{
+	int features;
+
+	if (!p->dsp) {
+		return;
+	}
+
+	features = ast_dsp_get_features(p->dsp);
+
+	if (digit) {
+		if (onoff) {
+			features |= DSP_FEATURE_DIGIT_DETECT;
+		} else {
+			features &= ~DSP_FEATURE_DIGIT_DETECT;
+		}
+	}
+
+	if (fax) {
+		if (onoff) {
+			features |= DSP_FEATURE_FAX_DETECT;
+		} else {
+			features &= ~DSP_FEATURE_FAX_DETECT;
+		}
+	}
+
+	ast_dsp_set_features(p->dsp, features);
+}
+
 static void disable_dsp_detect(struct sip_pvt *p)
 {
 	if (p->dsp) {
@@ -4672,11 +4702,16 @@
 			char *cp = (char *) data;
 
 			ast_debug(1, "%sabling digit detection on %s\n", *cp ? "En" : "Dis", chan->name);
-			if (*cp) {
-				enable_dsp_detect(p);
-			} else {
-				disable_dsp_detect(p);
-			}
+			update_dsp_detect(p, *cp, 1, 0);
+			res = 0;
+		}
+		break;
+	case AST_OPTION_FAX_DETECT:
+		if (ast_test_flag(&p->flags[1], SIP_PAGE2_FAX_DETECT_CNG)) {
+			char *cp = (char *) data;
+
+			ast_debug(1, "%sabling fax detection on %s\n", *cp ? "En" : "Dis", chan->name);
+			update_dsp_detect(p, *cp, 0, 1);
 			res = 0;
 		}
 		break;
@@ -4736,8 +4771,9 @@
 		break;
 	case AST_OPTION_DIGIT_DETECT:
 		cp = (char *) data;
-		*cp = p->dsp ? 1 : 0;
+		*cp = (ast_dsp_get_features(p->dsp) & DSP_FEATURE_DIGIT_DETECT) ? 1 : 0;
 		ast_debug(1, "Reporting digit detection %sabled on %s\n", *cp ? "en" : "dis", chan->name);
+		res = 0;
 		break;
 	case AST_OPTION_SECURE_SIGNALING:
 		*((unsigned int *) data) = p->req_secure_signaling;
@@ -30053,7 +30089,8 @@
 	if (p->rtp)
 		ast_rtp_instance_set_prop(p->rtp, AST_RTP_PROPERTY_DTMF, ast_test_flag(&p->flags[0], SIP_DTMF) == SIP_DTMF_RFC2833);
 	if ((ast_test_flag(&p->flags[0], SIP_DTMF) == SIP_DTMF_INBAND) ||
-	    (ast_test_flag(&p->flags[0], SIP_DTMF) == SIP_DTMF_AUTO)) {
+	    (ast_test_flag(&p->flags[0], SIP_DTMF) == SIP_DTMF_AUTO) ||
+	    ast_test_flag(&p->flags[1], SIP_PAGE2_FAX_DETECT_CNG)) {
 		enable_dsp_detect(p);
 	} else {
 		disable_dsp_detect(p);
--- include/asterisk/dsp.h	2009-10-21 03:09:04.000000000 +0000
+++ include/asterisk/dsp.h	2013-02-20 11:02:21.244897355 +0000
@@ -115,6 +115,9 @@
 /*! \brief Select feature set */
 void ast_dsp_set_features(struct ast_dsp *dsp, int features);
 
+/*! \brief Return feature set */
+int ast_dsp_get_features(struct ast_dsp *dsp);
+
 /*! \brief Get pending DTMF/MF digits */
 int ast_dsp_getdigits(struct ast_dsp *dsp, char *buf, int max);
 
--- main/dsp.c	2012-10-04 20:15:35.000000000 +0000
+++ main/dsp.c	2013-02-20 12:27:12.408135771 +0000
@@ -1619,6 +1619,11 @@
 	}
 }
 
+int ast_dsp_get_features(struct ast_dsp *dsp)
+{
+	return dsp ? dsp->features : 0;
+}
+
 void ast_dsp_free(struct ast_dsp *dsp)
 {
 	ast_free(dsp);
